#!/bin/bash

set -e
set -u
set -x

### Build patched juju 1.18.1 from the repo ###

sudo apt-get install -y bzr debhelper gccgo-go
mkdir -p ~/juju && cd $_
bzr export juju-core-1.18.1-fix lp:~ce-hyperscale/+junk/juju-core
cd juju-core-1.18.1-fix/
dpkg-buildpackage -b
cd ..
sudo dpkg -i juju_1.18.1-0ubuntu1_all.deb juju-core_1.18.1-0ubuntu1_arm64.deb

### Prepare juju environment ###

mkdir -p ~/juju-env && cd $_
juju generate-config -f

cat << EOF > ~/.juju/environments.yaml
default: arm0429demo
environments:
  arm0429demo:
    auth-mode: userpass
    auth-url: http://master1a:5000/v2.0/
    control-bucket: juju-control-bucket-5327bd70-9401-455b-b907-5b79591f781f
    default-series: trusty
    image-metadata-url: http://master1a/juju-meta/images/
    password: admin
    region: regionOne
    tools-metadata-url: http://streams.canonical.com/juju/tools/
    type: openstack
    use-floating-ip: 'false'
    username: admin
  stage:
    auth-mode: userpass
    auth-url: http://master1a:5000/v2.0/
    control-bucket: juju-control-bucket-4553b757-16a5-4a07-a416-0f87fb1688e9
    default-series: trusty
    image-metadata-url: http://master1a/juju-meta/images/
    password: admin
    region: regionOne
    tools-metadata-url: http://streams.canonical.com/juju/tools/
    type: openstack
    use-floating-ip: 'false'
    username: admin
  backup:
    auth-mode: userpass
    auth-url: http://master1a:5000/v2.0/
    control-bucket: juju-control-bucket-cc4e3563-eaba-467f-b643-b6c784be4e27
    default-series: trusty
    image-metadata-url: http://master1a/juju-meta/images/
    password: admin
    region: regionOne
    tools-metadata-url: http://streams.canonical.com/juju/tools/
    type: openstack
    use-floating-ip: 'false'
    username: admin
EOF

### Bake image metadata and share it with apache ###

sudo apt-get install -y apache2
export META_PATH="/var/www/html/juju-meta"
sudo mkdir -p $META_PATH
sudo chown `whoami` $META_PATH
export UBUNTU_IMAGE_ID=`glance image-list | grep "ubuntu " | tr -d ' ' | cut -d'|' -f2`
juju metadata generate-image -a arm64 -r 14.04 -d $META_PATH -i $UBUNTU_IMAGE_ID -r regionOne
chmod -R a+r $META_PATH

### Run juju bootstrap ###

juju bootstrap --upload-tools --metadata-source="http://master1a/juju-meta/images/" --constraints "arch=arm64 root-disk=5"

### Install wordpress and mysql ###

mkdir -p ~/charms/trusty/ && cd $_
bzr export wordpress lp:~ce-hyperscale/charms/precise/wordpress/arm64-trusty
bzr export mysql lp:~ce-hyperscale/charms/precise/mysql/arm64-trusty
juju deploy local:mysql --repository="/home/`whoami`/charms/"
juju deploy local:wordpress --repository="/home/`whoami`/charms/"
juju add-relation wordpress mysql
juju expose wordpress
