#!/bin/bash

set -e
set -u
set -x

# Do the following change to /etc/network/interfaces:
#  iface eth0 inet static
#    address <ip-of-the-node>
#    netmask 255.255.255.0
#    gateway 172.16.10.1
#    dns-nameservers 172.16.10.1
#    dns-search demo
# Reboot your machine with sudo reboot

# Run the following commands to disable THP
#  sudo apt-get install -y software-properties-common
#  sudo add-apt-repository -y ppa:ce-hyperscale/jujudemo
#  sudo apt-get update
#  sudo apt-get dist-upgrade -y
#  sudo flash-kernel
# Reboot your machine with sudo reboot

export MASTER_NODE_NAME="master1a"

sudo apt-get install -y mysql-client python-mysqldb nova-compute-kvm nova-network nova-api-metadata crudini

sudo rm /var/lib/nova/nova.sqlite
cat /dev/null | sudo tee /etc/nova/nova-compute.conf

if ! grep --quiet novatap /etc/network/interfaces; then
sudo sh -c "printf \"
# This interface is required by nova-compute
auto novatap
iface novatap inet manual
\tpre-up ip tuntap add dev \\\$IFACE mode tap
\tup ip link set \\\$IFACE up
\tdown ip link set \\\$IFACE down
\tdown ip tuntap del dev \\\$IFACE mode tap
\" >> /etc/network/interfaces"
sudo ifup novatap
fi

export MYIP=`ifconfig eth0 | grep 'inet addr:' | cut -d: -f2 | awk '{print $1}'`

sudo crudini --set /etc/nova/nova.conf DEFAULT connection_type libvirt
sudo crudini --set /etc/nova/nova.conf DEFAULT compute_driver libvirt.LibvirtDriver
sudo crudini --set /etc/nova/nova.conf DEFAULT quota_instances 1000
sudo crudini --set /etc/nova/nova.conf DEFAULT quota_security_groups 1000
sudo crudini --set /etc/nova/nova.conf DEFAULT vnc_enabled false
sudo crudini --set /etc/nova/nova.conf DEFAULT glance_host $MASTER_NODE_NAME

#sudo crudini --set /etc/nova/nova.conf DEFAULT public_interface eth0
#sudo crudini --set /etc/nova/nova.conf DEFAULT flat_interface novatap

sudo crudini --set /etc/nova/nova.conf DEFAULT public_interface novatap
sudo crudini --set /etc/nova/nova.conf DEFAULT flat_interface eth0

sudo crudini --set /etc/nova/nova.conf DEFAULT flat_network_bridge br100
sudo crudini --set /etc/nova/nova.conf DEFAULT network_manager nova.network.manager.FlatDHCPManager
sudo crudini --set /etc/nova/nova.conf DEFAULT firewall_driver nova.virt.libvirt.firewall.IptablesFirewallDriver
sudo crudini --set /etc/nova/nova.conf DEFAULT multi_host True
sudo crudini --set /etc/nova/nova.conf DEFAULT my_ip $MYIP
sudo crudini --set /etc/nova/nova.conf DEFAULT auth_strategy keystone
sudo crudini --set /etc/nova/nova.conf DEFAULT auth_protocol http
sudo crudini --set /etc/nova/nova.conf DEFAULT rpc_backend nova.rpc.impl_kombu
sudo crudini --set /etc/nova/nova.conf DEFAULT rabbit_host $MASTER_NODE_NAME
sudo crudini --set /etc/nova/nova.conf DEFAULT rabbit_password RABBIT_PASS
sudo crudini --set /etc/nova/nova.conf DEFAULT metadata_host 127.0.0.1
sudo crudini --set /etc/nova/nova.conf DEFAULT metadata_port 8775

sudo crudini --set /etc/nova/nova.conf database connection mysql://nova:NOVA_DBPASS@$MASTER_NODE_NAME/nova

sudo crudini --set /etc/nova/nova.conf libvirt virt_type kvm
sudo crudini --set /etc/nova/nova.conf libvirt cpu_mode host-passthrough

sudo crudini --set /etc/nova/nova.conf keystone_authtoken auth_host $MASTER_NODE_NAME
sudo crudini --set /etc/nova/nova.conf keystone_authtoken auth_port 35357
sudo crudini --set /etc/nova/nova.conf keystone_authtoken auth_protocol http
sudo crudini --set /etc/nova/nova.conf keystone_authtoken admin_tenant_name service
sudo crudini --set /etc/nova/nova.conf keystone_authtoken admin_user nova
sudo crudini --set /etc/nova/nova.conf keystone_authtoken admin_password NOVA_PASS

sudo crudini --set /etc/nova/api-paste.ini filter:authtoken paste.filter_factory keystoneclient.middleware.auth_token:filter_factory
sudo crudini --set /etc/nova/api-paste.ini filter:authtoken auth_host $MASTER_NODE_NAME
sudo crudini --set /etc/nova/api-paste.ini filter:authtoken auth_uri http://$MASTER_NODE_NAME:5000
sudo crudini --set /etc/nova/api-paste.ini filter:authtoken admin_tenant_name service
sudo crudini --set /etc/nova/api-paste.ini filter:authtoken admin_user nova
sudo crudini --set /etc/nova/api-paste.ini filter:authtoken admin_password NOVA_PASS
sudo crudini --set /etc/nova/api-paste.ini filter:authtoken auth_version v2.0

sudo service nova-api-metadata restart
sudo service nova-network restart
sudo service nova-compute restart
