#!/bin/bash

set -e
set -u
set -x

# Do the following change to /etc/network/interfaces:
#  iface eth0 inet static
#    address <ip-of-the-node>
#    netmask 255.255.255.0
#    gateway 172.16.10.1
#    dns-nameservers 172.16.10.1
#    dns-search demo
# Reboot your machine with sudo reboot

# Run the following commands to disable THP
#  sudo apt-get install -y software-properties-common
#  sudo add-apt-repository -y ppa:ce-hyperscale/jujudemo
#  sudo apt-get update
#  sudo apt-get dist-upgrade -y
#  sudo flash-kernel
# Reboot your machine with sudo reboot

export MASTER_NODE_NAME="master1a"
export ADMIN_TOKEN=`openssl rand -hex 10`

### Installing crudini ###

sudo apt-get install -y crudini

### Installing RabbitMQ ###

sudo apt-get install -y rabbitmq-server
sudo rabbitmqctl change_password guest RABBIT_PASS

### Installing MySQL ###

sudo debconf-set-selections <<< 'mysql-server-5.5 mysql-server/root_password password root'
sudo debconf-set-selections <<< 'mysql-server-5.5 mysql-server/root_password_again password root'
sudo apt-get install -y mysql-server mysql-client python-mysqldb

sudo sed -i '/bind-address/d' /etc/mysql/my.cnf

sudo tee << EOF /etc/mysql/conf.d/mysqld_utf8.cnf
[mysqld]
collation-server = utf8_general_ci
init-connect = 'SET NAMES utf8'
character-set-server = utf8
EOF

sudo service mysql restart

### Keystone (installation) ###

sudo apt-get install -y keystone
sudo rm /var/lib/keystone/keystone.db

sudo crudini --set /etc/keystone/keystone.conf database connection mysql://keystone:KEYSTONE_DBPASS@$MASTER_NODE_NAME/keystone
sudo crudini --set /etc/keystone/keystone.conf DEFAULT admin_token $ADMIN_TOKEN

mysql -uroot -proot << EOF
CREATE DATABASE keystone;
GRANT ALL PRIVILEGES ON keystone.* TO 'keystone'@'localhost' IDENTIFIED BY 'KEYSTONE_DBPASS';
GRANT ALL PRIVILEGES ON keystone.* TO 'keystone'@'%' IDENTIFIED BY 'KEYSTONE_DBPASS';
EOF

sudo keystone-manage db_sync
sudo service keystone restart

export OS_SERVICE_TOKEN=$ADMIN_TOKEN
export OS_SERVICE_ENDPOINT=http://$MASTER_NODE_NAME:35357/v2.0

sleep 5

keystone tenant-create --name=admin --description="Admin Tenant"
keystone tenant-create --name=service --description="Service Tenant"
keystone user-create --name=admin --pass=admin
keystone role-create --name=admin
keystone user-role-add --user=admin --tenant=admin --role=admin
keystone service-create --name=keystone --type=identity --description="Keystone Identity Service"

export KEYSTONE_ID=`keystone service-list | grep keystone | tr -d ' ' | cut -d'|' -f2`
keystone endpoint-create --service-id=$KEYSTONE_ID \
                         --publicurl=http://$MASTER_NODE_NAME:5000/v2.0 \
                         --internalurl=http://$MASTER_NODE_NAME:5000/v2.0 \
                         --adminurl=http://$MASTER_NODE_NAME:35357/v2.0

### Keystone (verification) ###

sleep 5
unset OS_SERVICE_TOKEN OS_SERVICE_ENDPOINT
keystone --os-username=admin --os-password=admin --os-auth-url=http://$MASTER_NODE_NAME:35357/v2.0 token-get
keystone --os-username=admin --os-password=admin --os-tenant-name=admin --os-auth-url=http://$MASTER_NODE_NAME:35357/v2.0 token-get

### Update .bashrc ###

cat << EOF >> ~/.bashrc

# Openstack credentials
export OS_USERNAME=admin
export OS_PASSWORD=admin
export OS_TENANT_NAME=admin
export OS_AUTH_URL=http://$MASTER_NODE_NAME:35357/v2.0
EOF

export OS_USERNAME=admin
export OS_PASSWORD=admin
export OS_TENANT_NAME=admin
export OS_AUTH_URL=http://$MASTER_NODE_NAME:35357/v2.0

keystone token-get
keystone user-list

### Glance (installation) ###

sudo apt-get install -y glance
sudo crudini --set /etc/glance/glance-registry.conf database connection mysql://glance:GLANCE_DBPASS@$MASTER_NODE_NAME/glance
sudo crudini --set /etc/glance/glance-api.conf database connection mysql://glance:GLANCE_DBPASS@$MASTER_NODE_NAME/glance

mysql -uroot -proot << EOF
CREATE DATABASE glance;
GRANT ALL PRIVILEGES ON glance.* TO 'glance'@'localhost' IDENTIFIED BY 'GLANCE_DBPASS';
GRANT ALL PRIVILEGES ON glance.* TO 'glance'@'%' IDENTIFIED BY 'GLANCE_DBPASS';
EOF

sudo glance-manage db_sync

keystone user-create --name=glance --pass=GLANCE_PASS
keystone user-role-add --user=glance --tenant=service --role=admin

sudo crudini --set /etc/glance/glance-api.conf keystone_authtoken auth_host $MASTER_NODE_NAME
sudo crudini --set /etc/glance/glance-api.conf keystone_authtoken auth_port 35357
sudo crudini --set /etc/glance/glance-api.conf keystone_authtoken auth_protocol http
sudo crudini --set /etc/glance/glance-api.conf keystone_authtoken admin_tenant_name service
sudo crudini --set /etc/glance/glance-api.conf keystone_authtoken admin_user glance
sudo crudini --set /etc/glance/glance-api.conf keystone_authtoken admin_password GLANCE_PASS

sudo crudini --set /etc/glance/glance-registry.conf keystone_authtoken auth_host $MASTER_NODE_NAME
sudo crudini --set /etc/glance/glance-registry.conf keystone_authtoken auth_port 35357
sudo crudini --set /etc/glance/glance-registry.conf keystone_authtoken auth_protocol http
sudo crudini --set /etc/glance/glance-registry.conf keystone_authtoken admin_tenant_name service
sudo crudini --set /etc/glance/glance-registry.conf keystone_authtoken admin_user glance
sudo crudini --set /etc/glance/glance-registry.conf keystone_authtoken admin_password GLANCE_PASS

sudo crudini --set /etc/glance/glance-api-paste.ini filter:authtoken paste.filter_factory keystoneclient.middleware.auth_token:filter_factory
sudo crudini --set /etc/glance/glance-api-paste.ini filter:authtoken auth_host $MASTER_NODE_NAME
sudo crudini --set /etc/glance/glance-api-paste.ini filter:authtoken admin_user glance
sudo crudini --set /etc/glance/glance-api-paste.ini filter:authtoken admin_tenant_name service
sudo crudini --set /etc/glance/glance-api-paste.ini filter:authtoken admin_password GLANCE_PASS

sudo crudini --set /etc/glance/glance-registry-paste.ini filter:authtoken paste.filter_factory keystoneclient.middleware.auth_token:filter_factory
sudo crudini --set /etc/glance/glance-registry-paste.ini filter:authtoken auth_host $MASTER_NODE_NAME
sudo crudini --set /etc/glance/glance-registry-paste.ini filter:authtoken admin_user glance
sudo crudini --set /etc/glance/glance-registry-paste.ini filter:authtoken admin_tenant_name service
sudo crudini --set /etc/glance/glance-registry-paste.ini filter:authtoken admin_password GLANCE_PASS

keystone service-create --name=glance --type=image --description="Glance Image Service"
export GLANCE_ID=`keystone service-list | grep glance | tr -d ' ' | cut -d'|' -f2`
keystone endpoint-create --service-id=$GLANCE_ID \
                         --publicurl=http://$MASTER_NODE_NAME:9292 \
                         --internalurl=http://$MASTER_NODE_NAME:9292 \
                         --adminurl=http://$MASTER_NODE_NAME:9292

sudo service glance-registry restart
sudo service glance-api restart

### Glance (verification) ###

sleep 5
mkdir -p /tmp/openstack-script && cd $_
wget http://cdn.download.cirros-cloud.net/0.3.1/cirros-0.3.1-x86_64-disk.img
glance image-create --name=cirros-0.3.1 --disk-format=qcow2 --container-format=bare --is-public=true < cirros-0.3.1-x86_64-disk.img
glance image-delete cirros-0.3.1

### Nova (installation) ###

sudo apt-get install -y python-novaclient nova-novncproxy novnc nova-api nova-ajax-console-proxy nova-cert nova-conductor nova-consoleauth nova-doc nova-scheduler nova-network

sudo rm /var/lib/nova/nova.sqlite

mysql -uroot -proot << EOF
CREATE DATABASE nova;
GRANT ALL PRIVILEGES ON nova.* TO 'nova'@'localhost' IDENTIFIED BY 'NOVA_DBPASS';
GRANT ALL PRIVILEGES ON nova.* TO 'nova'@'%' IDENTIFIED BY 'NOVA_DBPASS';
EOF

if ! grep --quiet novatap /etc/network/interfaces; then
sudo sh -c "printf \"
# This interface is required by nova-compute
auto novatap
iface novatap inet manual
\tpre-up ip tuntap add dev \\\$IFACE mode tap
\tup ip link set \\\$IFACE up
\tdown ip link set \\\$IFACE down
\tdown ip tuntap del dev \\\$IFACE mode tap
\" >> /etc/network/interfaces"
sudo ifup novatap
fi

export MYIP=`ifconfig eth0 | grep 'inet addr:' | cut -d: -f2 | awk '{print $1}'`

sudo crudini --set /etc/nova/nova.conf DEFAULT connection_type libvirt
sudo crudini --set /etc/nova/nova.conf DEFAULT compute_driver libvirt.LibvirtDriver
sudo crudini --set /etc/nova/nova.conf DEFAULT vnc_enabled false
sudo crudini --set /etc/nova/nova.conf DEFAULT glance_host $MASTER_NODE_NAME

sudo crudini --set /etc/nova/nova.conf DEFAULT quota_instances -1
sudo crudini --set /etc/nova/nova.conf DEFAULT quota_cores -1
sudo crudini --set /etc/nova/nova.conf DEFAULT quota_ram -1
sudo crudini --set /etc/nova/nova.conf DEFAULT quota_floating_ips -1
sudo crudini --set /etc/nova/nova.conf DEFAULT quota_fixed_ips -1
sudo crudini --set /etc/nova/nova.conf DEFAULT quota_metadata_items -1
sudo crudini --set /etc/nova/nova.conf DEFAULT quota_injected_files -1
sudo crudini --set /etc/nova/nova.conf DEFAULT quota_injected_file_content_bytes -1
sudo crudini --set /etc/nova/nova.conf DEFAULT quota_injected_file_path_bytes -1
sudo crudini --set /etc/nova/nova.conf DEFAULT quota_key_pairs -1
sudo crudini --set /etc/nova/nova.conf DEFAULT quota_security_groups -1
sudo crudini --set /etc/nova/nova.conf DEFAULT quota_security_group_rules -1

#sudo crudini --set /etc/nova/nova.conf DEFAULT public_interface eth0
#sudo crudini --set /etc/nova/nova.conf DEFAULT flat_interface novatap

sudo crudini --set /etc/nova/nova.conf DEFAULT public_interface novatap
sudo crudini --set /etc/nova/nova.conf DEFAULT flat_interface eth0

sudo crudini --set /etc/nova/nova.conf DEFAULT flat_network_bridge br100
sudo crudini --set /etc/nova/nova.conf DEFAULT network_manager nova.network.manager.FlatDHCPManager
sudo crudini --set /etc/nova/nova.conf DEFAULT firewall_driver nova.virt.libvirt.firewall.IptablesFirewallDriver
sudo crudini --set /etc/nova/nova.conf DEFAULT multi_host True
sudo crudini --set /etc/nova/nova.conf DEFAULT my_ip $MYIP
sudo crudini --set /etc/nova/nova.conf DEFAULT auth_strategy keystone
sudo crudini --set /etc/nova/nova.conf DEFAULT auth_protocol http
sudo crudini --set /etc/nova/nova.conf DEFAULT rpc_backend nova.rpc.impl_kombu
sudo crudini --set /etc/nova/nova.conf DEFAULT rabbit_host $MASTER_NODE_NAME
sudo crudini --set /etc/nova/nova.conf DEFAULT rabbit_password RABBIT_PASS
sudo crudini --set /etc/nova/nova.conf DEFAULT metadata_host 127.0.0.1
sudo crudini --set /etc/nova/nova.conf DEFAULT metadata_port 8775
#sudo crudini --set /etc/nova/nova.conf DEFAULT default_floating_pool public

sudo crudini --set /etc/nova/nova.conf database connection mysql://nova:NOVA_DBPASS@$MASTER_NODE_NAME/nova

sudo crudini --set /etc/nova/nova.conf libvirt virt_type kvm
sudo crudini --set /etc/nova/nova.conf libvirt cpu_mode host-passthrough

sudo crudini --set /etc/nova/nova.conf keystone_authtoken auth_host $MASTER_NODE_NAME
sudo crudini --set /etc/nova/nova.conf keystone_authtoken auth_port 35357
sudo crudini --set /etc/nova/nova.conf keystone_authtoken auth_protocol http
sudo crudini --set /etc/nova/nova.conf keystone_authtoken admin_tenant_name service
sudo crudini --set /etc/nova/nova.conf keystone_authtoken admin_user nova
sudo crudini --set /etc/nova/nova.conf keystone_authtoken admin_password NOVA_PASS

sudo crudini --set /etc/nova/api-paste.ini filter:authtoken paste.filter_factory keystoneclient.middleware.auth_token:filter_factory
sudo crudini --set /etc/nova/api-paste.ini filter:authtoken auth_host $MASTER_NODE_NAME
sudo crudini --set /etc/nova/api-paste.ini filter:authtoken auth_uri http://$MASTER_NODE_NAME:5000
sudo crudini --set /etc/nova/api-paste.ini filter:authtoken admin_tenant_name service
sudo crudini --set /etc/nova/api-paste.ini filter:authtoken admin_user nova
sudo crudini --set /etc/nova/api-paste.ini filter:authtoken admin_password NOVA_PASS
sudo crudini --set /etc/nova/api-paste.ini filter:authtoken auth_version v2.0

sudo nova-manage db sync

keystone user-create --name=nova --pass=NOVA_PASS
keystone user-role-add --user=nova --tenant=service --role=admin
keystone service-create --name=nova --type=compute --description="Nova Compute service"
export NOVA_ID=`keystone service-list | grep nova | tr -d ' ' | cut -d'|' -f2`
keystone endpoint-create --service-id=$NOVA_ID \
                         --publicurl=http://$MASTER_NODE_NAME:8774/v2/%\(tenant_id\)s \
                         --internalurl=http://$MASTER_NODE_NAME:8774/v2/%\(tenant_id\)s \
                         --adminurl=http://$MASTER_NODE_NAME:8774/v2/%\(tenant_id\)s

sudo service nova-api restart
sudo service nova-cert restart
sudo service nova-consoleauth restart
sudo service nova-scheduler restart
sudo service nova-conductor restart
sudo service nova-novncproxy restart
sudo service nova-network restart

sleep 5
nova flavor-delete m1.tiny
nova flavor-delete m1.small
nova flavor-delete m1.medium
nova flavor-delete m1.large
nova flavor-delete m1.xlarge

### Nova (verification) ###

sleep 5
nova image-list

### Nova Compute (installation) ###

sudo apt-get install -y nova-compute-kvm

ssh-keygen -t rsa -N "" -f ~/.ssh/id_rsa
nova keypair-add --pub_key ~/.ssh/id_rsa.pub mykey

nova flavor-create m1.ubuntu auto 2000 0 1
nova flavor-create m1.5GB auto 2000 5 1

sudo service nova-api restart
sudo service nova-cert restart
sudo service nova-consoleauth restart
sudo service nova-scheduler restart
sudo service nova-conductor restart
sudo service nova-novncproxy restart
sudo service nova-network restart
sudo service nova-compute restart

sleep 5

nova network-create vmnet --fixed-range-v4=172.16.10.255/26 --bridge-interface=br100 --multi-host=T

#nova network-create vmnet --fixed-range-v4=10.0.128.0/24 --bridge-interface=br100 --multi-host=T
#sudo nova-manage floating create --ip_range=172.16.10.255/26 --pool public

nova secgroup-add-rule default icmp -1 -1 0.0.0.0/0
nova secgroup-add-rule default tcp 22 22 0.0.0.0/0

### Swift (installation) ###

sudo apt-get install -y swift memcached python-netifaces python-xattr python-memcache swift-account swift-container swift-object swift-proxy

sudo mkdir -p /etc/swift
sudo touch /etc/swift/swift.conf
sudo touch /etc/swift/proxy-server.conf
sudo chown -R swift:swift /etc/swift/
sudo mkdir -p /srv/node/local0
sudo chown -R swift:swift /srv/node
sudo mkdir -p /var/swift/recon
sudo chown -R swift:swift /var/swift/recon
sudo mkdir -p /home/swift/keystone-signing
sudo chown -R swift:swift /home/swift/keystone-signing

cd /etc/swift
sudo openssl req -new -x509 -nodes -subj "/C=US/ST=NONE/L=NONE/O=NONE/CN=NONE" -out cert.crt -keyout cert.key
sudo chown swift:swift cert.crt cert.key

export SWIFT_HASH_PREFIX=`od -t x8 -N 8 -A n < /dev/urandom`
sudo crudini --set /etc/swift/swift.conf swift-hash swift_hash_path_suffix $SWIFT_HASH_PREFIX

sudo crudini --set /etc/swift/proxy-server.conf DEFAULT bind_port 8888
sudo crudini --set /etc/swift/proxy-server.conf DEFAULT user swift
sudo crudini --set /etc/swift/proxy-server.conf pipeline:main pipeline "healthcheck cache authtoken keystoneauth proxy-server"
sudo crudini --set /etc/swift/proxy-server.conf app:proxy-server use egg:swift#proxy
sudo crudini --set /etc/swift/proxy-server.conf app:proxy-server allow_account_management true
sudo crudini --set /etc/swift/proxy-server.conf app:proxy-server account_autocreate true
sudo crudini --set /etc/swift/proxy-server.conf filter:keystoneauth use egg:swift#keystoneauth
sudo crudini --set /etc/swift/proxy-server.conf filter:keystoneauth operator_roles "Member,admin,swiftoperator"
sudo crudini --set /etc/swift/proxy-server.conf filter:authtoken paste.filter_factory keystoneclient.middleware.auth_token:filter_factory
sudo crudini --set /etc/swift/proxy-server.conf filter:authtoken delay_auth_decision true
sudo crudini --set /etc/swift/proxy-server.conf filter:authtoken signing_dir /home/swift/keystone-signing
sudo crudini --set /etc/swift/proxy-server.conf filter:authtoken auth_protocol http
sudo crudini --set /etc/swift/proxy-server.conf filter:authtoken auth_host $MYIP
sudo crudini --set /etc/swift/proxy-server.conf filter:authtoken auth_port 35357
sudo crudini --set /etc/swift/proxy-server.conf filter:authtoken admin_token $ADMIN_TOKEN
sudo crudini --set /etc/swift/proxy-server.conf filter:authtoken admin_tenant_name service
sudo crudini --set /etc/swift/proxy-server.conf filter:authtoken admin_user swift
sudo crudini --set /etc/swift/proxy-server.conf filter:authtoken admin_password SWIFT_PASS
sudo crudini --set /etc/swift/proxy-server.conf filter:cache use egg:swift#memcache
sudo crudini --set /etc/swift/proxy-server.conf filter:catch_errors use egg:swift#catch_errors
sudo crudini --set /etc/swift/proxy-server.conf filter:healthcheck use egg:swift#healthcheck

sudo crudini --set /etc/swift/account-server.conf DEFAULT mount_check false
sudo crudini --set /etc/swift/container-server.conf DEFAULT mount_check false
sudo crudini --set /etc/swift/object-server.conf DEFAULT mount_check false

sudo swift-ring-builder account.builder create 18 1 1
sudo swift-ring-builder container.builder create 18 1 1
sudo swift-ring-builder object.builder create 18 1 1

sudo swift-ring-builder account.builder add z1-$MYIP:6002/local0 100
sudo swift-ring-builder container.builder add z1-$MYIP:6001/local0 100
sudo swift-ring-builder object.builder add z1-$MYIP:6000/local0 100

sudo swift-ring-builder account.builder rebalance
sudo swift-ring-builder container.builder rebalance
sudo swift-ring-builder object.builder rebalance

sudo chown swift:swift /etc/swift/*.ring.gz

keystone user-create --name=swift --pass=SWIFT_PASS
keystone user-role-add --user=swift --tenant=service --role=admin

sudo service swift-proxy restart

keystone service-create --name=swift --type=object-store --description="Object Storage Service"
export SWIFT_ID=`keystone service-list | grep swift | tr -d ' ' | cut -d'|' -f2`
keystone endpoint-create --service-id=$SWIFT_ID --publicurl "http://$MASTER_NODE_NAME:8888/v1/AUTH_%(tenant_id)s" --internalurl "http://$MASTER_NODE_NAME:8888/v1/AUTH_%(tenant_id)s" --adminurl "http://$MASTER_NODE_NAME:8888/v1"

sudo swift-init main start
sudo service memcached restart

### Swift (verification) ###

sleep 5
mkdir -p /tmp/openstack-script && cd $_

od -t x8 -N 32 -A n < /dev/urandom > my-file-1.txt
od -t x8 -N 32 -A n < /dev/urandom > my-file-2.txt

swift upload my-container my-file-1.txt
swift upload my-container my-file-2.txt
rm my-file-*
swift download my-container
swift delete my-container

### NTP server ###

sudo apt-get install -y ntp
sudo sed -i '/server/d' /etc/ntp.conf
echo "server 127.127.1.0" | sudo tee -a /etc/ntp.conf
echo "fudge 127.127.1.0 stratum 10" | sudo tee -a /etc/ntp.conf
sudo service ntp restart

### Horizon (installation) ###

sudo apt-get install -y openstack-dashboard
sudo apt-get remove -y --purge openstack-dashboard-ubuntu-theme

### Installing cloud image ###

case `uname -m` in
    "armv7l")
        UBUNTU_ARCH="armhf"
        ;;
    "aarch64")
        UBUNTU_ARCH="arm64"
        ;;
    *)
        UBUNTU_ARCH="arm64"
        echo "[Installing cloud image] Can't detect machine architecture, assuming ${UBUNTU_ARCH}..."
        ;;
esac

sudo apt-get install -y cloud-utils
mkdir -p ~/cloud-image/ && cd $_
wget http://cloud-images.ubuntu.com/trusty/current/trusty-server-cloudimg-${UBUNTU_ARCH}-disk1.img

sudo mount-image-callback trusty-server-cloudimg-${UBUNTU_ARCH}-disk1.img -- sh -c "cp \$MOUNTPOINT/boot/vmlinuz-* $PWD/vmlinuz"
sudo mount-image-callback trusty-server-cloudimg-${UBUNTU_ARCH}-disk1.img -- sh -c "sed -i \"s/\(NTPSERVERS *= *\).*/\1\\\"$MASTER_NODE_NAME\\\"/\" \$MOUNTPOINT/etc/default/ntpdate"
sudo mount-image-callback trusty-server-cloudimg-${UBUNTU_ARCH}-disk1.img -- sh -c "sed -i \"s/exit 0$/echo never | tee \/sys\/kernel\/mm\/transparent_hugepage\/enabled\nexit 0/\" \$MOUNTPOINT/etc/rc.local"

sudo chmod a+rw vmlinuz
glance image-create --name="ubuntu-kernel" --disk-format=aki --container-format=aki --is-public=true < vmlinuz
export KERNEL_IMAGE_ID=`glance image-list | grep ubuntu-kernel | tr -d ' ' | cut -d'|' -f2`

sudo qemu-nbd -r -c /dev/nbd0 trusty-server-cloudimg-${UBUNTU_ARCH}-disk1.img
sudo dd if=/dev/nbd0p1 of=image
sudo qemu-nbd -d /dev/nbd0

glance image-create --name="ubuntu" --disk-format=ami --container-format=ami --is-public=true --property kernel_id=$KERNEL_IMAGE_ID < image
